"""
Copyright 2020 BoardEx

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from . import global_var # import all the varaibles and libraries


if global_var.upload_to_sftp:
    from . import class_bx_sftp as bx_sftp  # import class_bx_sftp only if upload_to_sftp == True in config.py

# ----------------------------------------------------------------------------------------------------------------------
#  update BoardExID to salesforce
# ----------------------------------------------------------------------------------------------------------------------
class upload_to_salesfore:
    # ----------------------------------------------------------------------------------------------------------------------
    #  update contact to salesforce
    # ----------------------------------------------------------------------------------------------------------------------
    def update_contacts_to_sf(self, salesforce_connection, path_split_contact):

        global_var.logger.debug('update_contact_to_salesforce')
        global_var.logger.info('Please wait updating contacts to salesforce')

        bx_out_notnull = global_var.pd.read_csv(path_split_contact, encoding=global_var.encoder, sep = global_var.delimiter)
        df2 = global_var.pd.DataFrame(bx_out_notnull)

        total_rows = len(df2)
        counter = 0

        Bulk_data = []

        for i in (range(0,len(df2))):
            BoardExID = df2['BoardExID'].values[i]
            IBSID = df2['IBSID'].values[i]

            if  str(BoardExID) == 'nan' or str(BoardExID).strip() == '':
                global_var.logger.debug('ERROR: no BXid for ' + str(IBSID))
            else:
                data = {'Id': IBSID, 'BoardEx__Client_Individual_ID__c': BoardExID}
                Bulk_data.append(data)
        try:
            global_var.logger.info('Updating contacts to salesforce')
            t = salesforce_connection.bulk.Contact.upsert(Bulk_data, 'Id')
            global_var.logger.info('Update complete')


        except Exception as e:
                    global_var.logger.error(e, exc_info=True)
                    global_var.logger.debug('ERROR: Boardex ID not updated for  ' + IBSID )
                    raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  update company to salesforce
    # ----------------------------------------------------------------------------------------------------------------------
    def update_company_to_sf(self, salesforce_connection, path_split_contact):

        global_var.logger.debug('update_company_to_sf')
        global_var.logger.info('Please wait updating company salesforce')

        bx_out_notnull = global_var.pd.read_csv(path_split_contact, encoding=global_var.encoder, sep = global_var.delimiter)
        df2 = global_var.pd.DataFrame(bx_out_notnull)

        total_rows = len(df2)
        counter = 0

        Bulk_data = []


        for i in (range(0,len(df2))):
            CompanyID = str(df2['CompanyID'].values[i])
            ClientCompanyID = str(df2['ClientCompanyID'].values[i])

            if  str(CompanyID) == 'nan' or str(CompanyID).strip() == '' :
                global_var.logger.debug('ERROR: no BXid for ' + ClientCompanyID)
            else:
                data = {'Id': ClientCompanyID, 'BoardEx__Client_Organization_ID__c': CompanyID}

                Bulk_data.append(data)
        try:
            global_var.logger.info('Updating Company to salesforce')
            t = salesforce_connection.bulk.Account.upsert(Bulk_data, 'Id')
            global_var.logger.info('Update complete')

        except Exception as e:
                    global_var.logger.error(e, exc_info=True)
                    global_var.logger.debug('ERROR: Boardex ID not updated for  ' + ClientCompanyID )
                    raise e
# ----------------------------------------------------------------------------------------------------------------------
#  Download all csv OUT file if upload_to_sftp == True
# ----------------------------------------------------------------------------------------------------------------------
class download_from_sftp:
    ### The main methods are called from classes.class_bx_sftp.py
    def download_all_csv(self):
        if global_var.upload_to_sftp == True:
            # ----------------------------------------------------------------------------------------------------------------------
            #  sftp connection to BoardEx
            # ----------------------------------------------------------------------------------------------------------------------
            sftp_conn = bx_sftp.bx_sftp().bx_sftp_connection()
            # ----------------------------------------------------------------------------------------------------------------------
            #  Get all csv files on BoardEx sftp server
            # ----------------------------------------------------------------------------------------------------------------------
            all_remote_csv = bx_sftp.bx_sftp().get_all_csv(sftp_conn)

            if len(all_remote_csv) !=0 :
                # ----------------------------------------------------------------------------------------------------------------------
                #  download all csv files on BoardEx sftp server
                # ----------------------------------------------------------------------------------------------------------------------
                bx_sftp.bx_sftp().download_all_csv_from_bx(sftp_conn, all_remote_csv)
                # ----------------------------------------------------------------------------------------------------------------------
                #   create archive folder on BoardEx sftp server
                # ----------------------------------------------------------------------------------------------------------------------
                bx_sftp.create_folder().create_remote_archive_folder(sftp_conn)
                # ----------------------------------------------------------------------------------------------------------------------
                #   move files to archive folder on sftp server
                # ----------------------------------------------------------------------------------------------------------------------
                bx_sftp.move_files().move_files_to_archive(sftp_conn,all_remote_csv)
            else:
                global_var.logger.info('No csv OUT file found sftp server')
        else:
            global_var.logger.debug('upload_to_sftp : ' + str(global_var.upload_to_sftp))



"""
Copyright 2020 BoardEx

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

# ----------------------------------------------------------------------------------------------------------------------
# library import
# ----------------------------------------------------------------------------------------------------------------------

from datetime import date                                   # current date
from datetime import datetime
import time
import os, shutil                                           # Manipulates OS folder
import pprint                                               # Displays output more clearly
from boardex_salesforce.config import username, password, security_token, myHostname, myPassword, myUsername, path_local_temp, log_level, company_query, contact_query, UserDetails_query, upload_to_sftp, query_email, ContactDetails_query
from datetime import date                                   # current date
import pysftp                                               # sftp connection
from tqdm import tqdm                                       # progress bar
import pandas as pd
import csv
from simple_salesforce import Salesforce
import random, string
import logging                                              # "logs log"
import shutil




# ----------------------------------------------------------------------------------------------------------------------
# current date
# ----------------------------------------------------------------------------------------------------------------------

currentDate = str(datetime.today().strftime('%Y%m%d')) # creates current date (yyyymmdd) for csv file name


# ----------------------------------------------------------------------------------------------------------------------
# Local path
# ----------------------------------------------------------------------------------------------------------------------
local_split_folder = 'split_csv_files'
path_local_split_csv_files = local_split_folder
moved_combined_company_cvs_filepath = path_local_split_csv_files +'\\'+currentDate+"_company_combined_csv.csv"
moved_combined_contact_cvs_filepath = path_local_split_csv_files+'\\'+currentDate+"_contact_combined_csv.csv"

combined_company_cvs_filepath = currentDate+"_company_combined_csv.csv"
combined_contact_cvs_filepath = currentDate+"_contact_combined_csv.csv"


# ----------------------------------------------------------------------------------------------------------------------
# Global Variables
# ----------------------------------------------------------------------------------------------------------------------
create_folder_mode = 777
counter = 0                                 # counts the number files to be uploaded to BoardEx sftp server
array_remote_contact_csv_files = []
array_remote_company_csv_files = []

# ----------------------------------------------------------------------------------------------------------------------
# Name for local folder
# ----------------------------------------------------------------------------------------------------------------------
folder_archive_name = 'archive'                 # folder's name for archive on server
folder_log_name = 'log'                         # folder's name for log on server
folder_upload_name = 'upload'
folder_download_name = 'download'

# ----------------------------------------------------------------------------------------------------------------------
# BoardEx sftp path
# ----------------------------------------------------------------------------------------------------------------------
path_remote_log = 'log'                        # remote temporary path for logs
path_remote_archive = 'archive'                  # path for archive folder

# ----------------------------------------------------------------------------------------------------------------------
# save as locally
# ----------------------------------------------------------------------------------------------------------------------
randomAlphaNumeric = ''.join(random.choices(string.ascii_letters + string.digits, k=6))  # creates 6 random characters

boardex_log ='_BoardEx_sftp.log'
fileExtention = '.csv'

save_BoardExFile_contact = '_Contacts_IN'
save_BoardExFileType_company = '_Company_IN'
save_BoardExFileType_ContactDetails = '_ContactDetails_I_PRD'
save_BoardExFileType_UserDetails = '_UserDetails_I_PRD'


split_company_output_filename = 'company_split_%s.csv'
split_contact_output_filename = 'Contacts_split_%s.csv'

save_company_as = currentDate + save_BoardExFileType_company + fileExtention
save_company_to = path_local_temp  + folder_download_name +'//'+save_company_as

save_contact_as = currentDate + save_BoardExFile_contact + fileExtention
save_contact_to = path_local_temp  +folder_download_name +'//'+ save_contact_as

save_ContactDetails_as = currentDate + save_BoardExFileType_ContactDetails + fileExtention
save_ContactDetails_to = path_local_temp +folder_download_name +'//'+ save_ContactDetails_as

save_UserDetails_as = currentDate + save_BoardExFileType_UserDetails + fileExtention
save_UserDetails_to = path_local_temp + folder_download_name +'//'+save_UserDetails_as

# ----------------------------------------------------------------------------------------------------------------------
# save as on sftp
# ----------------------------------------------------------------------------------------------------------------------

remote_path_company = '\\'+ save_company_as
remote_path_contact = '\\'+ save_contact_as
remote_path_ContactDetails = '\\'+ save_ContactDetails_as
remote_path_UserDetails = '\\'+ save_UserDetails_as


# ----------------------------------------------------------------------------------------------------------------------
# search file name containing a string
# ----------------------------------------------------------------------------------------------------------------------
search_company ='Company'
search_contact = 'Contacts'
search_logs = ".log"
search_split_contact = 'Contacts_split'
search_split_company = 'company_split'
search_split_csv = 'split'
search_out = 'OUT'

# ----------------------------------------------------------------------------------------------------------------------
# CSV encorder
# ----------------------------------------------------------------------------------------------------------------------
encoder = 'utf-8-sig'
# ----------------------------------------------------------------------------------------------------------------------
# CSV delimiter
# ----------------------------------------------------------------------------------------------------------------------
delimiter = '|'
# ----------------------------------------------------------------------------------------------------------------------
# CSV row limit
# ----------------------------------------------------------------------------------------------------------------------
row_limit1= 10000

# ----------------------------------------------------------------------------------------------------------------------
# initialise array variable.
# ----------------------------------------------------------------------------------------------------------------------
array_company = []                              # list of company csv files
array_contact = []                              # list of contact csv files
array_csv_only = []                             # list of all csv files
array_company_for_upload = []                   # list of company's data. Data of one csv file into an array.
array_contact_for_upload = []                   # list of contacts's data. Data of one csv file into an array.

# ----------------------------------------------------------------------------------------------------------------------
# variable for today's date for logs ONLY
# ----------------------------------------------------------------------------------------------------------------------
today = date.today()                            # today's date
dateformat = "%Y-%b-%d"
# ----------------------------------------------------------------------------------------------------------------------
#  creates a local temp folder
# ----------------------------------------------------------------------------------------------------------------------
if path_local_temp:
    #path_local_temp is empty
    try:
        if not os.path.exists(path_local_temp):
            #create directory
            os.makedirs(path_local_temp)
    except Exception as e:
        raise e

# ----------------------------------------------------------------------------------------------------------------------
#  Create log file
# ----------------------------------------------------------------------------------------------------------------------

if log_level == 'NOTSET':
    set_log_level = 0 # NOTSET
elif  log_level == 'DEBUG':
    set_log_level = 10 # Debug
elif log_level == 'INFO':
    set_log_level = 20  # INFO
elif  log_level == 'WARNING':
    set_log_level = 30 # WARNING
elif log_level == 'ERROR':
    set_log_level = 40  # ERROR
elif  log_level == 'CRITICAL':
    set_log_level = 50 # CRITICAL
else:
    set_log_level = 50  # CRITICAL


#def getlog(set_log_level)
logger = logging.getLogger(__name__)
logger.setLevel(set_log_level)

date_formatted = today.strftime(dateformat)
# create a file handler
log_filename = str(path_local_temp) + '\\' + str(date_formatted) + str(boardex_log)

try:
    if path_local_temp:
        # path_local_temp is empty
        handler = logging.FileHandler(log_filename)
    else:
        log_filename = str(date_formatted) + str(boardex_log)
        handler = logging.FileHandler(log_filename)
except Exception as e:
        raise e

handler.setLevel(set_log_level)

# create a logging format
formatter = logging.Formatter('%(asctime)s - %(name)s - %(funcName)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
# add the file handler to the logger
logger.addHandler(handler)



"""
Copyright 2020 BoardEx

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from . import global_var # import all the varaibles and libraries

# ----------------------------------------------------------------------------------------------------------------------
#  CSV
# ----------------------------------------------------------------------------------------------------------------------

class csv:
    # ----------------------------------------------------------------------------------------------------------------------
    #  Count number of rows in csv file
    # ----------------------------------------------------------------------------------------------------------------------
    def count_total_rows(self, filename):
            global_var.logger.debug('Counting total rows')
            with open(filename) as f:
                return sum(1 for line in f)

    # ----------------------------------------------------------------------------------------------------------------------
    #  Get all csv files on local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def get_csv_out(self):
        try:
            global_var.logger.debug('Getting all csv files local machine')
            listOfFile = global_var.os.listdir(global_var.folder_upload_name)
            global_var.logger.debug('Searching for files that contains the word: ' + str(
                global_var.search_out) + ' and has the extension ' + str(global_var.fileExtention))

            for file in listOfFile:
                if str(global_var.search_out) in file and str(file).endswith(str(global_var.fileExtention)):
                    global_var.array_csv_only.append(file)

            global_var.logger.info('csv out file : ' + str(global_var.array_csv_only))
            return global_var.array_csv_only

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  Get all company csv out files on local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def get_company_out(self, all_concordance_result_file):
        try:
            global_var.logger.debug('Getting all company csv files local machine')

            for company in all_concordance_result_file:
                if global_var.search_company in company:
                    global_var.array_company.append(str(global_var.folder_upload_name)+'//'+str(company))

            return global_var.array_company
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  Get all company csv out files on local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def get_contact_out(self,all_concordance_result_file):
        try:
            global_var.logger.debug('Getting all contact csv files local machine')

            for contact in all_concordance_result_file:
                if global_var.search_contact in contact:
                    global_var.array_contact.append(str(global_var.folder_upload_name)+'//'+str(contact))

            return global_var.array_contact
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  merge csv files
    # ----------------------------------------------------------------------------------------------------------------------
    def merge_csv(self, concordance_output, combined_cvs_filepath):

        try:
            global_var.logger.debug('Combining csv : ' + str(concordance_output))
            combined_csv = global_var.pd.concat(
            [global_var.pd.read_csv(f) for f in global_var.tqdm(concordance_output)], sort=False)

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e

        try:
            global_var.logger.debug('outputing csv :' + str(combined_cvs_filepath))

            combined_csv.to_csv(combined_cvs_filepath, index=False, encoding=global_var.encoder)
            global_var.logger.debug('Combine completed, file@ ' + str(combined_cvs_filepath))

            return combined_cvs_filepath

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  gets local split csv files
    # ----------------------------------------------------------------------------------------------------------------------
    def get_local_split_csv(self):
        global_var.logger.debug('Getting all csv files on local machine @ ' + str(global_var.path_local_split_csv_files))
        array_csv_only1 = []
        try:
        # Getting all csv files on local machine
        # r=root, d=directories, f = files
            for r, d, f in global_var.os.walk(global_var.path_local_split_csv_files):
                for file in f:
                    try:
                        if  global_var.search_split_csv in file and str(file).endswith('.csv') :
                            array_csv_only1.append(file)
                        else:
                            global_var.logger.debug('no split csv files found')

                    except Exception as e:
                        global_var.logger.error(e, exc_info=True)
                        raise e

            return array_csv_only1
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  gets local company split csv files
    # ----------------------------------------------------------------------------------------------------------------------
    def get_local_company_csv(self, all_local_csv):
        array_local_company = []
        global_var.logger.debug('Getting all company csv files on local machine from '
                                + str(global_var.path_local_split_csv_files))
        try:
            for eachFile in all_local_csv:
                if global_var.search_split_company in eachFile:
                    array_local_company.append(eachFile)

            return array_local_company
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e


    # ----------------------------------------------------------------------------------------------------------------------
    #  gets local contact split csv files
    # ----------------------------------------------------------------------------------------------------------------------
    def get_local_contact_csv(self, all_local_csv):
        array_local_contacts = []
        global_var.logger.debug('Getting all contact csv files on local machine from '
                                + str(global_var.path_local_split_csv_files))
        try:
            for eachFile in all_local_csv:
                if global_var.search_split_contact in eachFile:
                    array_local_contacts.append(eachFile)

            return array_local_contacts

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e

# ----------------------------------------------------------------------------------------------------------------------
#  split combine csv file into multiple files
# ----------------------------------------------------------------------------------------------------------------------
class split_csv:
    # ----------------------------------------------------------------------------------------------------------------------
    #  split company csv files
    # ----------------------------------------------------------------------------------------------------------------------
    def split_company_csv(self,  filehandler,
                          delimiter=global_var.delimiter,
                          row_limit=global_var.row_limit1,
                          output_name_template=global_var.split_company_output_filename,
                          output_path=global_var.path_local_split_csv_files,
                          keep_headers=True):

            global_var.logger.debug('Splitting company files and placing them in ' + str(global_var.path_local_split_csv_files))

            reader = global_var.csv.reader(filehandler, delimiter=delimiter)
            current_piece = 1
            current_out_path =global_var. os.path.join(
                output_path,
                output_name_template % current_piece
            )
            current_out_writer = global_var.csv.writer(open(current_out_path, 'w', newline='', encoding=global_var.encoder),
                                            delimiter=delimiter)
            current_limit = row_limit
            if keep_headers:
                headers = next(reader)
                current_out_writer.writerow(headers)
            for i, row in enumerate(reader):
                if i + 1 > current_limit:
                    current_piece += 1
                    current_limit = row_limit * current_piece
                    current_out_path = global_var.os.path.join(
                        output_path,
                      output_name_template % current_piece
                    )
                    current_out_writer = global_var.csv.writer(open(current_out_path, 'w', newline='', encoding=global_var.encoder),
                                                    delimiter=delimiter)
                    if keep_headers:
                        current_out_writer.writerow(headers)
                current_out_writer.writerow(row)

    # ----------------------------------------------------------------------------------------------------------------------
    #  split contact csv files
    # ----------------------------------------------------------------------------------------------------------------------

    def split_contact_csv(self, filehandler, delimiter='|', row_limit=global_var.row_limit1,
                          output_name_template=global_var.split_contact_output_filename,
                          output_path=global_var.path_local_split_csv_files, keep_headers=True):

        global_var.logger.debug('Splitting contact files and placing them in ' + str(global_var.path_local_split_csv_files))

        reader = global_var.csv.reader(filehandler, delimiter=delimiter)
        current_piece = 1
        current_out_path = global_var.os.path.join(
            output_path,
            output_name_template % current_piece
        )
        current_out_writer = global_var.csv.writer(open(current_out_path, 'w', newline='', encoding=global_var.encoder),
                                        delimiter=delimiter)
        current_limit = row_limit
        if keep_headers:
            headers = next(reader)
            current_out_writer.writerow(headers)
        for i, row in enumerate(reader):
            if i + 1 > current_limit:
                current_piece += 1
                current_limit = row_limit * current_piece
                current_out_path = global_var.os.path.join(
                    output_path,
                    output_name_template % current_piece
                )
                current_out_writer = global_var.csv.writer(open(current_out_path, 'w', newline='', encoding=global_var.encoder),
                                                delimiter=delimiter)
                if keep_headers:
                    current_out_writer.writerow(headers)
            current_out_writer.writerow(row)


"""
Copyright 2020 BoardEx

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from . import global_var # import all the varaibles and libraries

# ----------------------------------------------------------------------------------------------------------------------
#  Create sftp connection
# ----------------------------------------------------------------------------------------------------------------------
class bx_sftp:
    # ----------------------------------------------------------------------------------------------------------------------
    #  Create sftp connection
    # ----------------------------------------------------------------------------------------------------------------------
    def bx_sftp_connection(self):
        try:
            global_var.logger.debug('-----------------------------sftp_connection_to_BoardEx()'
                                    '------------------------------------------')
            global_var.logger.debug('connecting to Boardex Sftp service')

            sftp_connection = global_var.pysftp.Connection(global_var.myHostname,
                                                           username=global_var.myUsername,
                                                           password=global_var.myPassword)

            global_var.logger.debug(str(global_var.myUsername) + " is connected to: "
                                    + str(global_var.myHostname))
            global_var.logger.debug('Connected to Boardex Sftp service')

            return sftp_connection

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  Get all csv files on remote server
    # ----------------------------------------------------------------------------------------------------------------------
    def get_all_csv(self,sftp_connection):

        try:
            global_var.logger.debug('Getting all csv files from Boardex Sftp ')
            list_all_files = sftp_connection.listdir()
            global_var.logger.debug('Searching for files that contains the word: ' + str(
                global_var.search_out) + ' and has the extension ' + str(global_var.fileExtention))

            for file in list_all_files:
                if str(global_var.search_out) in file and str(file).endswith(str(global_var.fileExtention)):
                    global_var.array_csv_only.append(file)

            global_var.logger.info('csv out file : ' + str(global_var.array_csv_only))
            return global_var.array_csv_only
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e
    # ----------------------------- -----------------------------------------------------------------------------------------
    #  Download all csv file to local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def download_all_csv_from_bx(self,sftp_connection, all_csv_files):
        try:
            global_var.logger.debug('downloading all csv files from sftp server from BoardEx SFTP to '+ global_var.folder_upload_name)
            global_var.logger.debug( 'Downloading  '+str(len(all_csv_files)) +' files to '+ global_var.folder_upload_name)

            # ----------------------------------------------------------------------------------------------------------------------
            #  Create local folders
            # ----------------------------------------------------------------------------------------------------------------------
            local_dir().create_upload_folder()
            for item in  global_var.tqdm(all_csv_files):
                sftp_connection.get(str(item), str(global_var.folder_upload_name) +'//'+str(item))

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e

# ----------------------------------------------------------------------------------------------------------------------
#  Create  folders
# ----------------------------------------------------------------------------------------------------------------------
class create_folder:
    # ----------------------------------------------------------------------------------------------------------------------
    #  create remote archive
    # ----------------------------------------------------------------------------------------------------------------------
    def create_remote_archive_folder(self,sftp_connection):
        global_var.logger.debug('Checking for remote archive folder in archive')
        # check if global_var.path_remote_archive exist
        if sftp_connection.lexists(global_var.path_remote_archive):
            global_var.logger.debug('remote archive folder has already been created')
        else:
            global_var.logger.warning('no archive folder found')
            # creates global_var.path_remote_root folder
            try:
                global_var.logger.info('creating a remote archive folder')
                sftp_connection.mkdir(global_var.os.path.join(global_var.path_remote_archive),
                                      mode=global_var.create_folder_mode)
                global_var.logger.info('remote archive folder created')
            except Exception as e:
                global_var.logger.error(
                    'Could not create archive folder. ')
                global_var.logger.error(e, exc_info=True)
                raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  create log folder
    # ----------------------------------------------------------------------------------------------------------------------
    def create_remote_log_folder(self,sftp_connection ):
        global_var.logger.debug('Checking for remote log folder in archive')
        # check if global_var.path_remote_log exist
        if sftp_connection.lexists(global_var.path_remote_log):
            global_var.logger.debug('remote log folder has already been created')
        else:
            global_var.logger.warning('no log folder found')
            # creates global_var.path_remote_root folder
            try:
                global_var.logger.info('creating a remote log folder')
                sftp_connection.mkdir(global_var.os.path.join(global_var.path_remote_log),
                                      mode=global_var.create_folder_mode)
                global_var.logger.info('remote log folder created')
            except Exception as e:
                global_var.logger.error(
                    'Could not create log folder. ')
                global_var.logger.error(e, exc_info=True)
                raise e
# ----------------------------------------------------------------------------------------------------------------------
#  upload logs
# ----------------------------------------------------------------------------------------------------------------------
class upload_logs_to_sftp:
    # ----------------------------------------------------------------------------------------------------------------------
    #  upload logs to boardEx sftp server
    # ----------------------------------------------------------------------------------------------------------------------
    def upload_log(self,sftp_connection ):
        global_var.logger.debug('Listing all files in '+ str(global_var.path_local_temp))
        listOfFile = global_var.os.listdir()
        global_var.logger.debug('Files found '+ str(listOfFile))

        for logs in listOfFile:
            if global_var.search_logs in logs:
                global_var.logger.debug('Getting all log files ' + str(logs))
                remote_path = global_var.path_remote_log +'//'+ logs
                local_path = global_var.path_local_temp  + logs

                try:
                    global_var.logger.debug('Uploading log file to: ' + str(remote_path))
                    filePath = sftp_connection.put(local_path, remote_path)
                    global_var.logger.debug('log file uploaded to boardex sftp server: ' + remote_path)

                except Exception as e:
                   global_var.logger.error(e, exc_info=True)
                   raise e

# ----------------------------------------------------------------------------------------------------------------------
#  move files on sftp server
# ----------------------------------------------------------------------------------------------------------------------
class move_files:
    # ----------------------------------------------------------------------------------------------------------------------
    #  move files to remote archive folder
    # ----------------------------------------------------------------------------------------------------------------------
    def move_files_to_archive( self, sftp_connection, files):

            try:
                global_var.logger.debug('Moving: ' + str(files) + ' to archive folder on BX sftp')
                if len(files) != 0:
                    for file in files:
                        sftp_connection.rename(file, global_var.path_remote_archive+'//'+file)
                    global_var.logger.debug('Files moved to Archive folder on sftp completed')
                else:
                    global_var.logger.debug('no csv file found to move to archive folder')
            except Exception as e:
                global_var.logger.error(e, exc_info=True)
                raise e

    def move_local_csv_to_archive(self):
        global_var.logger.debug('move local csv to archive')

# ----------------------------------------------------------------------------------------------------------------------
#  creates folders on the local machine
# ----------------------------------------------------------------------------------------------------------------------
class local_dir:
    # ----------------------------------------------------------------------------------------------------------------------
    #  creates split  folder on the local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def create_splitfolder(self):
        try:
            global_var.logger.debug('Checking if there is a local split folder @ '+str(global_var.path_local_split_csv_files))
            if not global_var.os.path.exists(global_var.path_local_split_csv_files):
                global_var.os.makedirs(global_var.path_local_split_csv_files)
                global_var.logger.debug('split cvs folder has been created @ ' + global_var.path_local_split_csv_files)
            else:
                global_var.logger.info('There is split folder @ ' + global_var.path_local_split_csv_files)
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Could not create a local split folder @ ' + str(global_var.path_local_split_csv_files))
            raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  creates archive  folder on the local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def create_archive_folder(self):
        try:
            global_var.logger.debug('Checking if there is a local archive folder @ '+str(global_var.path_local_temp)+'\\'+str(global_var.folder_archive_name))
            if not global_var.os.path.exists(str(global_var.path_local_temp)+'\\'+str(global_var.folder_archive_name)):
                global_var.os.makedirs(str(global_var.path_local_temp)+'\\'+str(global_var.folder_archive_name))
                global_var.logger.debug('archive folder has been created @ ' +str(global_var.path_local_temp)+'\\'+str(global_var.folder_archive_name))
            else:
                global_var.logger.info('There is archive folder @ ' + str(global_var.path_local_temp)+'\\'+str(global_var.folder_archive_name))
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Could not create a local archive folder @ ' + str(global_var.path_local_temp)+'\\'+str(global_var.folder_archive_name))
            raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  creates upload  folder on the local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def create_upload_folder(self):
        try:
            global_var.logger.debug('Checking if there is a local upload folder @ '+str(global_var.folder_upload_name))
            if not global_var.os.path.exists(str(global_var.folder_upload_name)):
                global_var.os.makedirs(str(global_var.folder_upload_name))
                global_var.logger.debug('upload folder has been created @ ' +str(global_var.folder_upload_name))
            else:
                global_var.logger.info('There is upload folder @ ' +str(global_var.folder_upload_name))
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Could not create a local upload folder @ ' +str(global_var.folder_upload_name))
            raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  creates download  folder on the local machine
    # ----------------------------------------------------------------------------------------------------------------------
    def create_download_folder(self):
        try:
            global_var.logger.debug('Checking if there is a local download folder @ '+str(global_var.folder_download_name))
            if not global_var.os.path.exists(str(global_var.folder_download_name)):
                global_var.os.makedirs(str(global_var.folder_download_name))
                global_var.logger.debug('download folder has been created @ ' +str(global_var.folder_download_name))
            else:
                global_var.logger.info('There is download folder @ ' +str(global_var.folder_download_name))
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Could not create a local download folder @ ' +str(global_var.folder_download_name))
            raise e

class delete:
    def split_all_split_files(self):
        try:
            path_local_temp =global_var.os.listdir()

            for folder in path_local_temp:
                if global_var.local_split_folder in folder:
                    try:
                        global_var.logger.info('Deleting files in split folder')

                        global_var.shutil.rmtree(global_var.path_local_split_csv_files)
                    except Exception as e:
                        global_var.logger.error(e, exc_info=True)
                        raise e

        except Exception as e:
                    global_var.logger.error(e, exc_info=True)
                    global_var.logger.debug('ERROR: deleting split folder')

    def combined_csv(self):
        try:
            path_local_temp = global_var.os.listdir()

            for folder in path_local_temp:
                if global_var.combined_company_cvs_filepath in folder:
                    try:
                        global_var.logger.info('Deleting files in combined company folder')
                        global_var.os.remove(global_var.combined_company_cvs_filepath)
                    except Exception as e:
                        global_var.logger.error(e, exc_info=True)
                        raise e
                if global_var.combined_contact_cvs_filepath in folder:
                    try:
                        global_var.logger.info('Deleting files in combined contact folder')
                        global_var.os.remove(global_var.combined_contact_cvs_filepath)
                    except Exception as e:
                        global_var.logger.error(e, exc_info=True)
                        raise e


        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.debug('ERROR: deleting combined csv files')
            raise e



from .download_ContactDetails_from_sf import download_ContactDetails_from_sf
from .download_UserDetails_from_sf import download_UserDetails_from_sf

def run_daily():
    download_ContactDetails_from_sf()
    download_UserDetails_from_sf()

if __name__ == '__main__':
        run_daily()


from .download_company_from_sf import download_company_from_sf
from .download_contact_from_sf import download_contact_from_sf
from .download_ContactDetails_from_sf import download_ContactDetails_from_sf
from .download_UserDetails_from_sf import download_UserDetails_from_sf
from .upload_to_salesforce import upload_to_sf

def run_monthly():
    upload_to_sf()
    download_company_from_sf()
    download_contact_from_sf()
    download_ContactDetails_from_sf()
    download_UserDetails_from_sf()
if __name__ == '__main__':
        run_monthly()


"""
Copyright 2020 BoardEx

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from . import global_var # import all the varaibles and libraries


if global_var.upload_to_sftp:
     from . import class_bx_sftp as bx_sftp  # import class_bx_sftp only if upload_to_sftp == True in config.py

# ----------------------------------------------------------------------------------------------------------------------
#  SalesForce connection
# ----------------------------------------------------------------------------------------------------------------------
class salesforce_connection:
    # ----------------------------------------------------------------------------------------------------------------------
    #  create  salesforce connection
    # ----------------------------------------------------------------------------------------------------------------------
    def sf_connection(self):
        try:
            global_var.logger.info('##################------salesForce_connection()------#################')
            sfa = global_var.Salesforce(username=global_var.username, password=global_var.password,
                                        security_token=global_var.security_token)
            global_var.logger.info(str(global_var.username) + ' is connected to SalesForce')
            return sfa
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            if ('REQUEST_LIMIT_EXCEEDED' in str(e)):
                global_var.logger.error('REQUEST_LIMIT_EXCEEDED')
            else:
                global_var.logger.critical('Error: connection failed; please check username, password, security_token')
# ----------------------------------------------------------------------------------------------------------------------
#  makes sql queries to salesforce for companies and contacts
# ----------------------------------------------------------------------------------------------------------------------
class download_salesforce:
    # ----------------------------------------------------------------------------------------------------------------------
    #  sql query to salesforce - gets companies information
    # ----------------------------------------------------------------------------------------------------------------------
    def query_all_COMPANY_info(self, needSalesForceConnection):
        try:
            global_var.logger.info('Querying company details from SF')
            global_var.logger.debug('using the query: ' + str(global_var.company_query))

            accountsInfo = needSalesForceConnection.bulk.Account.query(global_var.company_query)
            global_var.logger.info(str(len(accountsInfo)) + ' records found FROM Account')

            return accountsInfo
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error(
                'Error: cannot query data from salesforce. Please check Credendtials or contact_query in the config.py file')
            raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  sql query to salesforce - gets contacts information
    # ----------------------------------------------------------------------------------------------------------------------
    def query_all_Contact_info(self, needSalesForceConnection):
        try:
            global_var.logger.info('Querying Contact from SF')
            if global_var.query_email == False:
                global_var.contact_query = "SELECT ID, FirstName, LastName, BoardEx__Client_Individual_ID__c, Title, Account.Name FROM Contact WHERE BoardEx__Client_Individual_ID__c !=''"
            else:
                global_var.contact_query

            global_var.logger.debug('using the query: ' + str(global_var.contact_query))

            accountsInfo = needSalesForceConnection.bulk.Contact.query(global_var.contact_query)

            global_var.logger.info(str(len(accountsInfo)) + ' records found FROM Contact')
            return accountsInfo
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error(
                'Error: cannot query data from salesforce. Please check Credendtials or contact_query in the config.py file')
            raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  sql query to salesforce - gets user details
    # ----------------------------------------------------------------------------------------------------------------------
    def query_all_UserDetails_info(self, needSalesForceConnection):
        try:
            global_var.logger.info('Querying User details from SF')
            global_var.logger.debug('using the query: ' + str(global_var.UserDetails_query))

            accountsInfo = needSalesForceConnection.bulk.Contact.query(global_var.UserDetails_query)

            global_var.logger.info(str(len(accountsInfo)) + ' records found FROM Contact')
            return accountsInfo
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error(
                'Error: cannot query data from salesforce. Please check Credendtials or UserDetails_query in the config.py file')
            raise e

    # ----------------------------------------------------------------------------------------------------------------------
    #  sql query to salesforce - gets contacts details
    # ----------------------------------------------------------------------------------------------------------------------
    def query_all_ContactDetails_info(self, needSalesForceConnection):
        try:
            global_var.logger.info('Querying User details from SF')
            global_var.logger.debug('using the query: ' + str(global_var.ContactDetails_query))

            accountsInfo = needSalesForceConnection.bulk.Contact.query(global_var.ContactDetails_query)

            global_var.logger.info(str(len(accountsInfo)) + ' records found FROM Contact')
            return accountsInfo
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error(
                'Error: cannot query data from salesforce. Please check Credendtials or ContactDetails_query in the config.py file')
            raise e
# ----------------------------------------------------------------------------------------------------------------------
#  Creates a table, adds BoardEx headers to the company and contact csv file
# ----------------------------------------------------------------------------------------------------------------------
class add_bx_headers:
    # ----------------------------------------------------------------------------------------------------------------------
    #  Compant table: The following headers will be added - Company, ClientCompanyID, ClientCountry,CompanyID
    # ----------------------------------------------------------------------------------------------------------------------
    def query_Company_into_dict(self, get_all_records):
        try:
            newList = []
            # Place query data into a data frame
            df = global_var.pd.DataFrame(get_all_records)
            global_var.logger.debug('Placing the company data from Salesforce in a table')
            df = df.loc[:, ['Name', 'Id', 'BillingCountry', 'BoardEx__Client_Organization_ID__c']]
            # Rename the extracted columns
            df.rename(columns={'Name': 'Company'}, inplace=True)
            df.rename(columns={'Id': 'ClientCompanyID'}, inplace=True)
            df.rename(columns={'BillingCountry': 'ClientCountry'}, inplace=True)
            df.rename(columns={'BoardEx__Client_Organization_ID__c': 'CompanyID'}, inplace=True)

            cols = list(df.columns.values)
            global_var.logger.debug('Creating the following Columns: ' + str(cols))
            # putting the data in an array
            arrayDF = df.to_numpy()
            global_var.logger.debug('Lenght of the array: ' + str(len(arrayDF)))
            global_var.logger.debug('Inserting the data from the array into a list')

            for item in global_var.tqdm(arrayDF):
                Company = str(item[0])
                ClientCompanyID = str(item[1])
                ClientCountry = str(item[2])
                CompanyID = str(item[3])

                newList.append(
                    {'Company': Company, 'ClientCompanyID': ClientCompanyID, 'ClientCountry': ClientCountry,
                     'CompanyID': CompanyID})

            global_var.logger.debug(str(len(newList)) + ' Compamies have been converted into BoardEx standard')
            return newList
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  add the following headers - Forename1, Forename2, Forename3, UsualName, Surname, IBSID, Role, Organisation, BoardExID
    # ----------------------------------------------------------------------------------------------------------------------
    def query_Contacts_into_dict(self, get_all_records):
                try:
                    newList = []
                    # Place query data into a data frame
                    df = global_var.pd.DataFrame(get_all_records)
                    global_var.logger.debug('Placing the Contact data from Salesforce in a table')
                    if global_var.query_email == False:
                        df = df.loc[:,
                         ['FirstName', 'LastName',  'Title', 'Account', 'Id', 'BoardEx__Client_Individual_ID__c']]
                    else:
                        df = df.loc[:,
                         ['FirstName', 'LastName', 'Email', 'Title', 'Account', 'Id', 'BoardEx__Client_Individual_ID__c']]
                    # Rename the columns
                    df.rename(columns={'FirstName': 'Forename1'}, inplace=True)
                    df.rename(columns={'LastName': 'Surname'}, inplace=True)
                    df.rename(columns={'Id': 'IBSID'}, inplace=True)
                    df.rename(columns={'Title': 'Role'}, inplace=True)
                    df.rename(columns={'BoardEx__Client_Individual_ID__c': 'BoardExID'}, inplace=True)
                    df.rename(columns={'Account': 'Organisation'}, inplace=True)

                    # Adds empty columns in specific location
                    df.insert(1, 'Forename2', '', allow_duplicates=False)
                    df.insert(2, 'Forename3', '', allow_duplicates=False)
                    df.insert(3, 'UsualName', '', allow_duplicates=False)

                    cols = list(df.columns.values)
                    global_var.logger.debug('Creating the following Columns: ' + str(cols))

                    # putting the data in an array
                    arrayDF = df.to_numpy()
                    global_var.logger.debug('Lenght of the array: ' + str(len(arrayDF)))
                    global_var.logger.debug('Inserting the data from the array into a list')

                    for item in global_var.tqdm(arrayDF):
                        Forename1 = str(item[0])
                        Forename2 = str(item[1])
                        Forename3 = str(item[2])
                        UsualName = str(item[3])
                        Surname = str(item[4])

                        if global_var.query_email == False:
                            Role = str(item[5])
                            Organisation = str(item[6]['Name'])
                            IBSID = str(item[7])
                            BoardExID = str(item[8])

                            newList.append(
                                {'Forename1': Forename1, 'Forename2': Forename2, 'Forename3': Forename3, 'UsualName': UsualName,
                                 'Surname': Surname, 'Role': Role, 'Organisation': Organisation, 'IBSID': IBSID,
                                 'BoardExID': BoardExID})
                        else:
                            Email = str(item[5])
                            Role = str(item[6])
                            Organisation = str(item[7]['Name'])
                            IBSID = str(item[8])
                            BoardExID = str(item[9])

                            newList.append(
                                {'Forename1': Forename1, 'Forename2': Forename2, 'Forename3': Forename3, 'UsualName': UsualName,
                                 'Surname': Surname, 'Email': Email, 'Role': Role, 'Organisation': Organisation, 'IBSID': IBSID,
                                 'BoardExID': BoardExID})


                    global_var.logger.debug(str(len(newList)) + ' Contacts have been converted into BoardEx standard')

                    return newList

                except Exception as e:
                    global_var.logger.error(e, exc_info=True)
                    raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  add the following headers - UserGUID, FirstName, Surname, Email, Department, Role, Region, NetworkBandID, Disabled
    # ----------------------------------------------------------------------------------------------------------------------
    def query_UserDetails_into_dict(self, get_all_records):
                try:
                    newList = []
                    # Place query data into a data frame
                    df = global_var.pd.DataFrame(get_all_records)
                    global_var.logger.debug('Placing the UserDetails data from Salesforce in a table')
                    df = df.loc[:,
                         ['FirstName', 'LastName', 'Email', 'Title']]

                    # Rename the extracted columns
                    df.rename(columns={'FirstName': 'FirstName'}, inplace=True)
                    df.rename(columns={'LastName': 'Surname'}, inplace=True)
                    df.rename(columns={'Email': 'Email'}, inplace=True)
                    df.rename(columns={'Title': 'Role'}, inplace=True)

                    # Adds empty columns in specific location
                    df.insert(0, 'UserGUID', '', allow_duplicates=False)
                    df.insert(4, 'Department', '', allow_duplicates=False)
                    df.insert(6, 'Region', '', allow_duplicates=False)
                    df.insert(7, 'NetworkBandID', '300', allow_duplicates=False)
                    df.insert(8, 'Disabled', '0', allow_duplicates=False)

                    cols = list(df.columns.values)
                    global_var.logger.debug('Creating the following Columns: ' + str(cols))

                    # putting the data in an array
                    arrayDF = df.to_numpy()
                    global_var.logger.debug('Lenght of the array: ' + str(len(arrayDF)))
                    global_var.logger.debug('Inserting the data from the array into a list')

                    for item in global_var.tqdm(arrayDF):
                        UserGUID = str(item[3])
                        FirstName = str(item[1])
                        Surname = str(item[2])
                        Email = str(item[3])
                        Department = str(item[4])
                        Role = str(item[5])
                        Region = str(item[6])
                        NetworkBandID = str(item[7])
                        Disabled = str(item[8])

                        newList.append(
                            {'UserGUID': UserGUID, 'FirstName': FirstName, 'Surname': Surname, 'Email': Email,
                             'Department': Department, 'Role': Role, 'Region': Region, 'NetworkBandID': NetworkBandID, 'Disabled': Disabled})

                    global_var.logger.debug(str(len(newList)) + ' UserDetails have been converted into BoardEx standard')

                    return newList

                except Exception as e:
                    global_var.logger.error(e, exc_info=True)
                    raise e
    # ----------------------------------------------------------------------------------------------------------------------
    #  add the following headers - UserGUID, FirstName, Surname, Email, Department, Role, Region, NetworkBandID, Disabled
    # ----------------------------------------------------------------------------------------------------------------------
    def query_ContactDetails_into_dict(self, get_all_records):
                try:
                    newList = []
                    userguid_entityid_only = [[]]
                    try:
                        get_totalSize = len(get_all_records)
                        for r in global_var.tqdm(range(0, get_totalSize)):
                            a = get_all_records[r]['Owner']['Email']
                            b = get_all_records[r]['BoardEx__Client_Individual_ID__c']
                            c = a, b
                            userguid_entityid_only.append(c)
                            # rename the columns
                            df = global_var.pd.DataFrame(userguid_entityid_only, columns=['UserGUID', 'EntityID'])
                    except Exception as e:
                        global_var.logger.error(e, exc_info=True)

                    # Place query data into a data frame
                    df = global_var.pd.DataFrame(df)

                    # Removes the first row which contains None for 'UserGUID', 'EntityID'
                    df = df.stack().str.replace('None', ' ').unstack()
                    global_var.logger.debug('Placing the ContactDetails data from Salesforce in a table')
                    df = df.loc[:,
                         ['UserGUID', 'EntityID']]

                    # Adds empty columns in specific location
                    df.insert(2, 'Weighting', '5', allow_duplicates=False)
                    df.insert(3, 'Personal', '', allow_duplicates=False)
                    df.insert(4, 'VisibilityID', '', allow_duplicates=False)
                    df.insert(5, 'PermissionID', '', allow_duplicates=False)
                    df.insert(6, 'Deleted', '', allow_duplicates=False)

                    cols = list(df.columns.values)
                    global_var.logger.debug('Creating the following Columns: ' + str(cols))

                    # putting the data in an array
                    arrayDF = df.to_numpy()
                    global_var.logger.debug('Lenght of the array: ' + str(len(arrayDF)))
                    global_var.logger.debug('Inserting the data from the array into a list')

                    for item in global_var.tqdm(arrayDF):
                        UserGUID = str(item[0])
                        EntityID = str(item[1])
                        Weighting = str(item[2])
                        Personal = str(item[3])
                        VisibilityID = str(item[4])
                        PermissionID = str(item[5])
                        Deleted = str(item[6])

                        newList.append(
                            {'UserGUID': UserGUID, 'EntityID': EntityID, 'Weighting': Weighting, 'Personal': Personal,
                             'VisibilityID': VisibilityID, 'PermissionID': PermissionID, 'Deleted': Deleted})

                    global_var.logger.debug(str(len(newList)) + ' ContactDetails have been converted into BoardEx standard')

                    return newList

                except Exception as e:
                    global_var.logger.error(e, exc_info=True)
                    raise e
# ----------------------------------------------------------------------------------------------------------------------
#  removes unauthorised characters
# ----------------------------------------------------------------------------------------------------------------------
class clean_data:
    def remove_unauthorised_characters(self,data_array):
        try:
            global_var.logger.debug('Converts array into a table')

            df = global_var.pd.DataFrame(data_array)

            global_var.logger.debug('Removes unauthorised characters ')

            # cleans data according to BoardEx Template
            df = df.stack().str.replace(',', ' ').unstack()
            df = df.stack().str.replace("'", ' ').unstack()
            df = df.stack().str.replace('|', ' ').unstack()
            df = df.stack().str.replace('(', ' ').unstack()
            df = df.stack().str.replace(')', ' ').unstack()
            df = df.stack().str.replace('\\', ' ').unstack()
            df = df.stack().str.replace('/', ' ').unstack()
            df = df.stack().str.replace('"', ' ').unstack()
            df = df.stack().str.replace('   ', ' ').unstack()
            df = df.stack().str.replace('None', ' ').unstack()
            return df
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            raise e
# ----------------------------------------------------------------------------------------------------------------------
#  export to csv file
# ----------------------------------------------------------------------------------------------------------------------
class export_to:
    # ----------------------------------------------------------------------------------------------------------------------
    #  export company csv file
    # ----------------------------------------------------------------------------------------------------------------------
    def company_csv(self,cleaned_company_data):

        # re-orders the columns hearders and their data.
        df = cleaned_company_data[['Company', 'ClientCompanyID', 'ClientCountry', 'CompanyID']]

        global_var.logger.debug('Saving file as: ' + str(global_var.save_company_as))
        global_var.logger.debug('Saving file to: ' + str(global_var.save_company_to))

        try:
            global_var.logger.info('Saving company csv as: ' + str(global_var.save_company_to))

            # Exports and encodes the file as a CSV file with a pipe delimiter
            df.to_csv(global_var.save_company_to, sep=global_var.delimiter, encoding=global_var.encoder, index=False)

            global_var.logger.info('Company csv saved to: ' + str(global_var.save_company_to))
        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Error saving Company data to csv')
            raise e

        # ----------------------------------------------------------------------------------------------------------------------
        #  uploads to csv to sftp
        # ----------------------------------------------------------------------------------------------------------------------
        if global_var.upload_to_sftp == True:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))
            try:
                export_to().BoardEx_sftp(global_var.save_company_to, global_var.remote_path_company)
            except Exception as e:
                global_var.logger.error(e, exc_info=True)
                raise e
        else:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))

    # ----------------------------------------------------------------------------------------------------------------------
    #  export contact csv file
    # ----------------------------------------------------------------------------------------------------------------------
    def contact_csv(self,cleaned_contact_data):
        # re-orders the columns hearders and their data.
        if global_var.query_email == False:
            df = cleaned_contact_data[['Forename1', 'Forename2', 'Forename3', 'UsualName', 'Surname', 'Role', 'Organisation', 'IBSID','BoardExID']]

        else:
            df = cleaned_contact_data[['Forename1', 'Forename2', 'Forename3', 'UsualName', 'Surname', 'Email', 'Role', 'Organisation', 'IBSID','BoardExID']]

        global_var.logger.debug('Saving file as: ' + str(global_var.save_contact_as))
        global_var.logger.debug('Saving file to: ' + str(global_var.save_contact_to))

        try:
            global_var.logger.info('Saving contact csv as: ' + str(global_var.save_contact_to))

            # Exports and encodes the file as a CSV file with a pipe delimiter
            df.to_csv(global_var.save_contact_to, sep=global_var.delimiter, encoding=global_var.encoder, index=False)
            global_var.logger.info('contact csv saved to: ' + str(global_var.save_contact_to))

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Error saving Company data to csv')
            raise e

        # ----------------------------------------------------------------------------------------------------------------------
        #  uploads to csv to sftp
        # ----------------------------------------------------------------------------------------------------------------------
        if global_var.upload_to_sftp == True:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))
            try:
                export_to().BoardEx_sftp(global_var.save_contact_to, global_var.remote_path_contact)
            except Exception as e:
                global_var.logger.error(e, exc_info=True)
                raise e
        else:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))

    # ----------------------------------------------------------------------------------------------------------------------
    #  export UserDetails csv file
    # ----------------------------------------------------------------------------------------------------------------------
    def UserDetails_csv(self,cleaned_contact_data):

        # re-orders the columns hearders and their data.
        df = cleaned_contact_data[
                ['UserGUID', 'FirstName', 'Surname', 'Email', 'Department', 'Role', 'Region', 'NetworkBandID',
                 'Disabled']]

        global_var.logger.debug('Saving file as: ' + str(global_var.save_UserDetails_as))
        global_var.logger.debug('Saving file to: ' + str(global_var.save_UserDetails_to))

        try:
            global_var.logger.info('Saving contact csv as: ' + str(global_var.save_UserDetails_to))

            # Exports and encodes the file as a CSV file with a pipe delimiter
            df.to_csv(global_var.save_UserDetails_to, sep=global_var.delimiter, encoding=global_var.encoder, index=False)

            global_var.logger.info('contact csv saved to: ' + str(global_var.save_UserDetails_to))

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Error saving Company data to csv')
            raise e

        # ----------------------------------------------------------------------------------------------------------------------
        #  uploads to csv to sftp
        # ----------------------------------------------------------------------------------------------------------------------
        if global_var.upload_to_sftp == True:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))
            try:
                export_to().BoardEx_sftp(global_var.save_UserDetails_to, global_var.remote_path_UserDetails)
            except Exception as e:
                global_var.logger.error(e, exc_info=True)
                raise e
        else:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))

    # ----------------------------------------------------------------------------------------------------------------------
    #  export ContactDetails csv file
    # ----------------------------------------------------------------------------------------------------------------------
    def ContactDetails_csv(self,cleaned_contact_data):
        # re-orders the columns hearders and their data.
        df = cleaned_contact_data[
                ['UserGUID', 'EntityID', 'Weighting', 'Personal', 'VisibilityID', 'PermissionID', 'Deleted']]

        global_var.logger.debug('Saving file as: ' + str(global_var.save_ContactDetails_as))
        global_var.logger.debug('Saving file to: ' + str(global_var.save_ContactDetails_to))

        try:
            global_var.logger.info('Saving contact csv as: ' + str(global_var.save_ContactDetails_to))

            # Exports and encodes the file as a CSV file with a pipe delimiter
            df.to_csv(global_var.save_ContactDetails_to, sep=global_var.delimiter, encoding=global_var.encoder, index=False)

            global_var.logger.info('contact csv saved to: ' + str(global_var.save_ContactDetails_to))

        except Exception as e:
            global_var.logger.error(e, exc_info=True)
            global_var.logger.error('Error saving Company data to csv')
            raise e

        # ----------------------------------------------------------------------------------------------------------------------
        #  uploads to csv to sftp
        # ----------------------------------------------------------------------------------------------------------------------
        if global_var.upload_to_sftp == True:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))
            try:
                export_to().BoardEx_sftp(global_var.save_ContactDetails_to, global_var.remote_path_ContactDetails)
            except Exception as e:
                global_var.logger.error(e, exc_info=True)
                raise e
        else:
            global_var.logger.info("upload_to_sftp: " + str(global_var.upload_to_sftp))

    # ----------------------------------------------------------------------------------------------------------------------
    #  connects to sftp server, creates the required folder and uploads csv file
    # ----------------------------------------------------------------------------------------------------------------------
    def BoardEx_sftp(self,local_csv, remote_csv):
        # ----------------------------------------------------------------------------------------------------------------------
        #  sftp connection to BoardEx from class_bx_sftp
        # ----------------------------------------------------------------------------------------------------------------------
            sftp_conn = bx_sftp.bx_sftp().bx_sftp_connection()
        # ----------------------------------------------------------------------------------------------------------------------
        #  creates a log folder on sftp server
        # ----------------------------------------------------------------------------------------------------------------------
            bx_sftp.create_folder().create_remote_log_folder(sftp_conn)
            global_var.logger.debug("Remote log folder created")
        # ----------------------------------------------------------------------------------------------------------------------
        #  creates an archive folder on sftp server
        # ----------------------------------------------------------------------------------------------------------------------
            bx_sftp.create_folder().create_remote_archive_folder(sftp_conn)
            global_var.logger.debug("Remote archive folder created")
        # ----------------------------------------------------------------------------------------------------------------------
        #  uploads csv file
        # ----------------------------------------------------------------------------------------------------------------------
            try:
                global_var.logger.debug("Uploading csv to sftp")
                filePath = sftp_conn.put(local_csv, remote_csv)
                global_var.logger.debug('File uploaded to: ' + str(remote_csv))
                global_var.logger.debug("Upload to sftp complete")

            except Exception as e:
                global_var.logger.error(e, exc_info=True)
                raise e

        # ----------------------------------------------------------------------------------------------------------------------
        #  uploads csv file
        # ----------------------------------------------------------------------------------------------------------------------

            global_var.logger.debug("Uploading logs")
            bx_sftp.upload_logs_to_sftp().upload_log(sftp_conn)
            global_var.logger.debug("Upload complete")




import setuptools

def readme():
    with open("README.md") as f:
        return f.read()

setuptools.setup(
    name="boardex_salesforce",
    version="0.0.1",
    author="BoardEx PTS",
    author_email="helpdesk@boardex.com",
    description="Downloads and Uploads to SalesForce from BoardEx stfp",
    long_description=readme(),
    long_description_content_type='text/markdown',
    url="https://aws-bxtfs01.global.root/tfs/BoardExCollection/PTS/_git/Salesforce%20Integration",
    packages = setuptools.find_packages(),
    license='MIT',
    install_requires =['simple_salesforce==1.10.1','pandas','pysftp==0.2.8','tqdm==4.48.0', 'pytest==6.0.1'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

    python_requires='>=3.5',
)